#test
xyz=123

#menu

file='File'
import_lib='Import Library'
lib_example='Example'
editor='Editor'
project='Project'
tool='Tool'
help='Use Help'

#menufile

openrecent='OpenRecent'
projectfile='PorjectFile'
open_folder='Open Project Folder'
exit='Exit App'
commenter='Comment / Cancel Comment'
language='LangSwitcher'
#toobar

complex='Compile'
upload='Compile and Download'
uploadfast='Download'
clean='Clean'
new='New'
open='Open'
save='Save'
saveall='Save All'
serial='Serial'
serialp='Serial pic'
search='Search'
back='Back'
forward='Forward'
editstyle='Page setup'
install_driver='Manual Installation Driver'
choose_down_type='Select Download Method'
usb_link='Serial port'
st_link='St_link V2'
tcommunity='Technical community'
community_web='Community Website'
about='About'
li_development='Library development'
create_lib='New Library'
search_lib='Search Library'
open_lib='Open Library Folder'
choose_board='Select board：'
choose_port='Download Port：'
check_port='Check Port'
restart_effect='Restart Settings Effective'

lib_search='Input Library Name'
lib_list='Library List'
board_list='New Pro'
example_list='Example'
project_list='Projects'
debug_button='Debug'
platform_list='Platforms'
package_list='Plug-ins'
package_uninstall='Uninstall'
lib_detail='More information'
lib_select='Select File to Install'
lib_version='Version information'
select_version='Latest version'
install_lib='Confirm installation'
lib_find_more='Find More'
stduino_lib_install='Sduino package Install'
no_net='No network to access, please connect the network and try again!'
format1='Format'
paste='Paste'
day_model='Daytime Model'
night_model='Night Model'
person_model='Personal Model'
person_setting='Options'
edit_back_color='Edit color'
main_back_color='Text color'
commenter_back_color='Comment color'
confirm_change='Confirm'
concle_change='Cancel'
exit_this='Exit'
find_next='Find/Next'
replace_next='Replace/Next'
replace_all='Replace All'
match='Exact match'
feedback='Feedback'
serial_port='No Serial Port!'
on=' On '
download_type_1=' Download Type: '

complexing='Compiling：'
start_complex='Begin to compile...'
now_program='The current program：'
row_line='Row/Column:'
warning_now_line='Warning: row/column:'
complex_complete='Compile successfully!'
complex_failed='Compilation fails!'
clean_out='Clear success'
clean_failed='Clear failed'
serial_port_check='Please check that the download port is selected correctly!'
serial_port_is_normal='Check whether the serial port is normal！'
st_link_check='Please check if the connection between St_link and MCU is correct!'
download_start='Start the download...'
downloading='Downloading：'
downloaded='Download successful!'
download_failed='Download failed!'
tip='Tip'
lib_installed='Library installation successful，\nThe library name that the actual import refers to is under src.h！'
lib_install_failed='Library installation failed, please try again later！'
lib_install_info='The library is unavailable and cannot be downloaded！'
lib_download='Downloading：'
shouce='The local instruction manual is being developed！'
server='We will gradually improve to provide you with better service'
server_detail='Problem description\nOpinion suggestion\nYour email address'
feed_back='Problem of feedback！'
feed_back_thank='Thank you for your feedback! Your comments will become an important basis for us to improve our software and services. We welcome your continued attention and support us！'
feed_back_failed='Feedback submission failed, please try again later'
about_ide='About this software'
function_development='This function is under development！'
lib_share='Select the library to share'
language_choose='Stduino IDE Library development language'
language_select='Select the library development language'
cancel='cancel'
lib_name='Library file name'
file_name="File name:"
line_text='//One keyword per line, delete the line after completion'
save_tip='Whether to save the changes'
save_tip_file='Whether to save the file'
last_close='This is the last window that will close the software'
find_text_none='Search completed: no such keyword!'
change_text_none='Replacement completed: no such keyword!'
change_text_num='Replacement completed: total replacement'
mail_sended='Email sent successfully'
mail_send_failed='Mail delivery failed, please try again later'
serial_port_helper='Serial port assistant'
serial_open_failed='This serial port cannot be opened！'
serial_port_staus_o='Serial port status (opened)'
serial_port_staus_c='Serial port status (closed)'
input_data_16='Please enter hexadecimal data separated by Spaces!'
send_hex='Sending HEX['
send_dec='Sending DEC['
receive_hex='Receiving HEX['
receive_dec='Receiving DEC['
reset_up='Restart the software to take effect'
startup_failed='Software startup failure'
search_tip='Please go to the official website to search“Resolution of software failure to start”！'
web_site='Official website：'
file_save='File saving'
select_file='Select the file'

add_file="Add file to projects"
add_direc="Add Directory to projects"
re_name="Rename..."
delete_f_d="Delete file/directory from projects"

code_flash = "Flash:"
code_ram = " TotalRam:"
code_static = " Percentage:"
code_bits = "Bits"

project_set="New project configuration"
project_name="Project name"
board_core_type="Chip"
project_framework="Framework"
board_usb="USB support"
board_usb_speed="USB speed"
board_opt="Compile settings"
board_runtime="Runtime library"
board_upload="Download mode"
board_debug="Debug mode"
project_build="Create"
library_installer="Exit the current installation interface"
search_sucess="Search Sucessfully!"