#test
xyz=123

#menu

file='文件'
import_lib='导入库'
lib_example='示例'
editor='编辑'
project='项目'
tool='工具'
help='教程'

#menufile

openrecent='打开最近的'
projectfile='项目文件夹'
open_folder='打开项目所在文件夹'
exit='关闭软件'
commenter='注释/取消注释'
language='语言切换'
#toobar

complex='编译'
upload='编译及下载'
uploadfast='下载'
clean='清除'
new='新建'
open='打开'
save='保存'
saveall='全部保存'
serial='串口监视器'
serialp='串口绘图器'
search='立即搜索'
back='回退'
forward='向前'
editstyle='页面设置'
install_driver='手动安装驱动'
choose_down_type='选择下载方式'
usb_link='串口下载'
st_link='St_link'
tcommunity='技术社区'
community_web='社区官网'
about='关于'
li_development='库开发'
create_lib='新建库'
search_lib='搜索库'
open_lib='打开库所在文件夹'
choose_board='选择板型：'
choose_port='选择下载端口：'
check_port='检测端口'
restart_effect='重启设置生效'

lib_search='输入库名称'
lib_list='库列表'
board_list='新项目'
example_list='示例'
project_list='项目'
debug_button='调试'
platform_list='平台'
package_list='插件'
package_uninstall='卸载'
lib_detail='更多信息'
lib_select='选择要安装的文件'
lib_version='版本信息'
select_version='当前版本'
install_lib='确认安装'
lib_find_more='查找更多'
stduino_lib_install='Sduino 包安装'
no_net='无网络可访问，请连网后重试!'
format1='格式化'
paste='粘贴'
day_model='白天模式'
night_model='夜晚模式'
person_model='个性模式'
person_setting='个性化设置选项'
edit_back_color='编辑背景颜色'
main_back_color='正文背景颜色'
commenter_back_color='注释背景颜色'
confirm_change='确认修改'
concle_change='取消修改'
exit_this='退出'
find_next='查找/下一个'
replace_next='替换/下一个'
replace_all='全部替换'
match='完全匹配'
feedback='反馈'
serial_port='无可用串口!'
on=' 在 '
download_type_1=' 当前下载方式： '

complexing='正在编译：'
start_complex='开始编译...'
now_program='当前程序：'
row_line='行/列:'
warning_now_line='警告：行/列:'
complex_complete='编译完成!'
complex_failed='编译失败!'
clean_out='清除完成'
clean_failed='清除失败'
serial_port_check='请检查下载端口是否选择正确或请拔掉0号引脚上的连接线!'
serial_port_is_normal='检查串口是否正常！'
st_link_check='请检查St_link与单片机间是否正确连接或者请更新st_link固件!'
download_start='开始下载...'
downloading='正在下载：'
downloaded='下载完成!'
download_failed='下载失败!'
tip='提示'
lib_installed='文件包安装成功！'
lib_install_failed='安装失败，请稍后重试！'
lib_install_info='该库无可用版本，暂时无法下载！'
lib_download='正在下载：'
shouce='本地使用说明手册正在开发ing！'
server='我们将逐步完善为您提供更好的服务'
server_detail='问题描述\n意见建议\n您的邮箱'
feed_back='问题反馈！'
feed_back_thank='感谢您的反馈!您的意见将成为我们改进软件，提升服务的重要依据,欢迎您继续关注支持我们！'
feed_back_failed='反馈提交失败，请稍后再试'
about_ide='关于本软件'
function_development='本功能正在开发！'
lib_share='选择要分享的库'
language_choose='Stduino IDE 库开发语言选择'
language_select='请选择库开发语言'
cancel='取消'
lib_name='库文件名'
file_name="文件名:"
line_text='//One keyword per line, delete the line after completion'
save_tip='被改动是否保存'
save_tip_file='是否保存该文件'
last_close='最后一个窗口将关闭软件'
find_text_none='查找完毕:无此关键词!'
change_text_none='替换完毕:无此关键词!'
change_text_num='替换完毕:共替换'
mail_sended='邮件发送成功'
mail_send_failed='邮件发送失败,请稍后再试'
serial_port_helper='串口助手'
serial_open_failed='此串口不能被打开！'
serial_port_staus_o='串口状态（已开启）'
serial_port_staus_c='串口状态（已关闭）'
input_data_16='请输入十六进制数据，以空格分开!'
send_hex='发HEX['
send_dec='发DEC['
receive_hex='收HEX['
receive_dec='收DEC['
reset_up='重启软件后生效'
startup_failed='软件启动失败'
search_tip='请至官网搜索“软件无法启动解决措施”！'
web_site='官方网站：'
file_save='文件保存'
select_file='选取文件'

add_file="添加文件至项目"
add_direc="添加项目文件夹"
re_name="重命名"
delete_f_d="删除文件/夹"

code_flash="代码总量:"
code_ram=" 总内存:"
code_static=" 所占百分比:"
code_bits="字节"

project_set="新建项目配置"
project_name="项目名称"
board_core_type="芯片"
project_framework="框架"
board_usb="USB支持"
board_usb_speed="USB速度"
board_opt="编译设置"
board_runtime="运行依赖库"
board_upload="下载方式"
board_debug="调试方式"
project_build="创建"
library_installer="退出当前安装界面"
search_sucess="搜索成功"